﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Be.Timvw.Framework.ComponentModel;
using RSN.Tools;

namespace RSN.DotP
{
	public partial class DeckInformation : Form
	{
		private Deck m_dkInfo;
		private Dictionary<int, string> m_dicUsedIds;
		private IdScheme m_isScheme;

		private bool m_bLoadingPersonality;
		private AiPersonality m_apPersonality;

		// For adjusting numeric up/downs and selected radio buttons (due to canceling)
		private bool m_bProgramaticAdjust;
		private bool m_bChangingToLast;

		// For building deck box images.
		private ImageBuilder m_ibDeckBox;
		private bool m_bBuildDeckImage;
		private RadioButton m_rbDeckLastChecked;

		// For building Personality images.
		private ImageBuilder m_ibSmallAvatar;
		private ImageBuilder m_ibLargeAvatar;
		private bool m_bBuildSmallAvatar;
		private bool m_bBuildLargeAvatar;
		private RadioButton m_rbSmallAvatarLastChecked;
		private RadioButton m_rbLargeAvatarLastChecked;

		public DeckInformation(Deck dkInfo, GameDirectory gdData)
		{
			InitializeComponent();

			m_dkInfo = dkInfo;
			m_dicUsedIds = gdData.UsedIds;

			// Load our current Id Scheme.
			m_isScheme = Settings.GetSerializableSetting("CurrentIdScheme", new IdScheme());
			numUid.Minimum = m_isScheme.MinimumId;
			numUid.Maximum = m_isScheme.MaximumId;

			LoadLocalizationStrings();

			Rectangle rcPosition = Settings.GetSetting("DeckInfoPosition", new Rectangle(-1, -1, -1, -1));
			if (rcPosition.X != -1)
			{
				bool bDoChecks = Settings.GetSetting("PerformBasicScreenChecks", true);
				if ((rcPosition.X >= 0) && (rcPosition.Y >= 0) &&
					(rcPosition.X < (Screen.PrimaryScreen.WorkingArea.X + Screen.PrimaryScreen.WorkingArea.Width - 50)) &&
					(rcPosition.Y < (Screen.PrimaryScreen.WorkingArea.Y + Screen.PrimaryScreen.WorkingArea.Height - 50)) || !bDoChecks)
				{
					this.StartPosition = FormStartPosition.Manual;
					this.Location = rcPosition.Location;
				}
				// This is a non-resizable form.
				//this.Size = rcPosition.Size;
			}

			// I have form sizing issues if I try to set this in the designer when coupled with a manual start position rather than from code.
			this.FormBorderStyle = FormBorderStyle.FixedDialog;

			// Now load our values.
			//	Basic
			cboAvailability.SelectedItem = new KeyValuePair<DeckAvailability, string>(m_dkInfo.Availability, Settings.UIStrings[m_dkInfo.Availability.ToString().ToUpper()]);
			foreach (DeckCard dcCard in m_dkInfo.Cards)
			{
				if (dcCard.Card.Filename.Equals(m_dkInfo.PreviewCard, StringComparison.OrdinalIgnoreCase))
				{
					cboPreviewCard.SelectedItem = dcCard;
					break;
				}
			}
			// Uid
			try
			{
				numUid.Value = m_dkInfo.Uid;
			}
			catch (Exception)
			{ }
			numUid_ValueChanged(null, null);

			// Deck Statistics
			if (!m_dkInfo.CreatureSize.Equals("?"))
				((RadioButton)pnlSize.Controls["rbSize" + m_dkInfo.CreatureSize]).Checked = true;
			if (!m_dkInfo.DeckSpeed.Equals("?"))
				((RadioButton)pnlSpeed.Controls["rbSpeed" + m_dkInfo.DeckSpeed]).Checked = true;
			if (!m_dkInfo.Flexibility.Equals("?"))
				((RadioButton)pnlFlexibility.Controls["rbFlex" + m_dkInfo.Flexibility]).Checked = true;
			if (!m_dkInfo.Synergy.Equals("?"))
				((RadioButton)pnlSynergy.Controls["rbSyn" + m_dkInfo.Synergy]).Checked = true;

			// Land Config
			numIgnoreCmcOver.Value = m_dkInfo.IgnoreCmcOver;
			numMinForests.Value = m_dkInfo.MinForests;
			numMinIslands.Value = m_dkInfo.MinIslands;
			numMinMountains.Value = m_dkInfo.MinMountains;
			numMinPlains.Value = m_dkInfo.MinPlains;
			numMinSwamps.Value = m_dkInfo.MinSwamps;
			numSpellsThatCountAsLand.Value = m_dkInfo.NumberOfSpellsThatCountAsLands;

			// Deck Box
			if (m_dkInfo.DeckBoxImage != null)
				picDeckBox.Image = new Bitmap(m_dkInfo.DeckBoxImage);
			rbUseDeckImage.Checked = true;
			rbUseDeckImage_CheckedChanged(null, null);
			rbBuildImage_CheckedChanged(null, null);

			// Personality
			m_bLoadingPersonality = true;
			cboMusic.DataSource = gdData.Music;
			cboPersonality.DisplayMember = "LocalizedName";
			if ((m_dkInfo.Personality != null) && (!m_dkInfo.Personality.BuiltIn))
			{
				SortableBindingList<AiPersonality> lstPersonalities = new SortableBindingList<AiPersonality>(gdData.Personalities.Values);
				lstPersonalities.Insert(0, m_dkInfo.Personality);
				cboPersonality.DataSource = lstPersonalities;
				cboPersonality.SelectedItem = m_dkInfo.Personality;
				m_apPersonality = new AiPersonality(m_dkInfo.Personality);
			}
			else
			{
				cboPersonality.DataSource = new SortableBindingList<AiPersonality>(gdData.Personalities.Values);
				if (m_dkInfo.Personality != null)
					cboPersonality.SelectedItem = m_dkInfo.Personality;
				m_apPersonality = m_dkInfo.Personality;
			}
			RefreshPersonality();

			// Make sure we can build deck box images.
			if ((!File.Exists(Settings.GetProgramDir() + "Images\\D13_DeckBox_Overlay.png")) ||
				(!File.Exists(Settings.GetProgramDir() + "Images\\D13_DeckBox_Mask.png")))
				rbBuildImage.Enabled = false;
			else
			{
				// Attempt to load them now so we don't have to later.
				try
				{
					Bitmap bmpLoadedOverlay = new Bitmap(Settings.GetProgramDir() + "Images\\D13_DeckBox_Overlay.png");
					Bitmap bmpLoadedMask = new Bitmap(Settings.GetProgramDir() + "Images\\D13_DeckBox_Mask.png");
					if ((bmpLoadedOverlay != null) && (bmpLoadedMask != null))
						m_ibDeckBox = new ImageBuilder(bmpLoadedOverlay, bmpLoadedMask, new Rectangle(49, 33, 157, 214));
					else
						rbBuildImage.Enabled = false;
				}
				catch (Exception e)
				{
					// We can't load the box template and mask so we can't build deck box images.
					rbBuildImage.Enabled = false;
					Settings.ReportError(e, ErrorPriority.Low, "Could not load Deck Box Overlay and Mask, building of deck box images will be disabled.");
				}
			}

			// Make sure we can build large avatars for personality.
			if ((!File.Exists(Settings.GetProgramDir() + "Images\\D13_LargeAvatar_Overlay.png")) ||
				(!File.Exists(Settings.GetProgramDir() + "Images\\D13_LargeAvatar_Mask.png")))
				rbBuildImageLarge.Enabled = false;
			else
			{
				// Attempt to load them now so we don't have to later.
				try
				{
					Bitmap bmpLoadedOverlay = new Bitmap(Settings.GetProgramDir() + "Images\\D13_LargeAvatar_Overlay.png");
					Bitmap bmpLoadedMask = new Bitmap(Settings.GetProgramDir() + "Images\\D13_LargeAvatar_Mask.png");
					if ((bmpLoadedOverlay != null) && (bmpLoadedMask != null))
					{
						m_ibLargeAvatar = new ImageBuilder(bmpLoadedOverlay, bmpLoadedMask, new Rectangle(51, 82, 429, 735));
						// Since we are showing the image at half width, half height we should adjust our mouse change by a factor of 2.
						m_ibLargeAvatar.MouseChangeModifier = 2f;
					}
					else
						rbBuildImageLarge.Enabled = false;
				}
				catch (Exception e)
				{
					// We can't load the box template and mask so we can't build deck box images.
					rbBuildImageLarge.Enabled = false;
					Settings.ReportError(e, ErrorPriority.Low, "Could not load Large Avatar Overlay and Mask, building of large avatar images will be disabled.");
				}
			}
			rbBuildImageLarge_CheckedChanged(null, null);

			// Make sure we can build small avatars for personality.
			if (!File.Exists(Settings.GetProgramDir() + "Images\\D13_SmallAvatar_Mask.png"))
				rbBuildImageSmall.Enabled = false;
			else
			{
				// Attempt to load the mask so we don't have to later.
				try
				{
					Bitmap bmpSmallAvatarMask = new Bitmap(Settings.GetProgramDir() + "Images\\D13_SmallAvatar_Mask.png");
					if (bmpSmallAvatarMask != null)
						m_ibSmallAvatar = new ImageBuilder(null, bmpSmallAvatarMask, new Rectangle(38, 38, 180, 180));
					else
						rbBuildImageSmall.Enabled = false;
				}
				catch (Exception e)
				{
					// Couldn't load the mask so we can't build small avatar images.
					rbBuildImageSmall.Enabled = false;
					Settings.ReportError(e, ErrorPriority.Low, "Could not load Small Avatar Mask, building of small avatar images will be disabled.");
				}
			}
			rbBuildImageSmall_CheckedChanged(null, null);

			m_bProgramaticAdjust = false;
		}

		private void RefreshPersonality()
		{
			m_bLoadingPersonality = true;
			lblPersonalityName.Text = m_apPersonality.LocalizedName;
			if ((m_apPersonality.MusicMix != null) && (m_apPersonality.MusicMix.Length > 0))
				cboMusic.SelectedItem = m_apPersonality.MusicMix;
			if (picPersonalitySmall.Image != null)
				picPersonalitySmall.Image.Dispose();
			if (m_apPersonality.SmallAvatarImage != null)
				picPersonalitySmall.Image = new Bitmap(m_apPersonality.SmallAvatarImage);
			else
				picPersonalitySmall.Image = null;
			if (picPersonalityLarge.Image != null)
				picPersonalityLarge.Image.Dispose();
			if (m_apPersonality.LargeAvatarImage != null)
				picPersonalityLarge.Image = new Bitmap(m_apPersonality.LargeAvatarImage);
			else
				picPersonalityLarge.Image = null;
			rbUseThisImageSmall.Checked = true;
			rbUseThisImageLarge.Checked = true;
			m_bLoadingPersonality = false;
		}

		private void PrepPersonalityForEditing()
		{
			// Only really need to care if this is a built-in personality, if custom it already is marked.
			if (m_apPersonality.BuiltIn)
			{
				// To really do this properly we need to create a new Personality as a copy of the current one
				m_bLoadingPersonality = true;
				m_apPersonality = new AiPersonality(m_apPersonality);
				// Add this to the drop down's data source and select it.
				((SortableBindingList<AiPersonality>)cboPersonality.DataSource).Add(m_apPersonality);
				cboPersonality.SelectedItem = m_apPersonality;
				m_bLoadingPersonality = false;
				// We should now be ready to accept any editing without screwing up the built-in personalities for this run.
			}
		}

		private void LoadLocalizationStrings()
		{
			this.Text = Settings.UIStrings[(string)this.Tag];

			// Basic Info
			gbBasicInfo.Text = Settings.UIStrings[(string)gbBasicInfo.Tag];
			lblAvailability.Text = Settings.UIStrings[(string)lblAvailability.Tag];
			cboAvailability.DisplayMember = "Value";
			cboAvailability.ValueMember = "Key";
			cboAvailability.Items.Add(new KeyValuePair<DeckAvailability, string>(DeckAvailability.AlwaysAvailable, Settings.UIStrings[DeckAvailability.AlwaysAvailable.ToString().ToUpper()]));
			cboAvailability.Items.Add(new KeyValuePair<DeckAvailability, string>(DeckAvailability.Locked, Settings.UIStrings[DeckAvailability.Locked.ToString().ToUpper()]));
			cboAvailability.Items.Add(new KeyValuePair<DeckAvailability, string>(DeckAvailability.NeverAvailable, Settings.UIStrings[DeckAvailability.NeverAvailable.ToString().ToUpper()]));
			lblPreviewCard.Text = Settings.UIStrings[(string)lblPreviewCard.Tag];
			cboPreviewCard.DisplayMember = "LocalizedCardName";
			cboPreviewCard.DataSource = m_dkInfo.Cards;
			lblUid.Text = Settings.UIStrings[(string)lblUid.Tag];
			cmdGetFirstAvailableUid.Text = Settings.UIStrings[(string)cmdGetFirstAvailableUid.Tag];
			lblFinalUidText.Text = Settings.UIStrings[(string)lblFinalUidText.Tag];

			// Deck Statistics
			gbDeckStatistics.Text = Settings.UIStrings[(string)gbDeckStatistics.Tag];
			lblCreatureSize.Text = Settings.UIStrings[(string)lblCreatureSize.Tag];
			lblSpeed.Text = Settings.UIStrings[(string)lblSpeed.Tag];
			lblFlexibility.Text = Settings.UIStrings[(string)lblFlexibility.Tag];
			lblSynergy.Text = Settings.UIStrings[(string)lblSynergy.Tag];

			// Land Config
			gbLandConfig.Text = Settings.UIStrings[(string)gbLandConfig.Tag];
			lblIgnoreCmcOver.Text = Settings.UIStrings[(string)lblIgnoreCmcOver.Tag];
			lblMinForests.Text = Settings.UIStrings[(string)lblMinForests.Tag];
			lblMinIslands.Text = Settings.UIStrings[(string)lblMinIslands.Tag];
			lblMinMountains.Text = Settings.UIStrings[(string)lblMinMountains.Tag];
			lblMinPlains.Text = Settings.UIStrings[(string)lblMinPlains.Tag];
			lblMinSwamps.Text = Settings.UIStrings[(string)lblMinSwamps.Tag];
			lblNumSpellsThatCountAsLand.Text = Settings.UIStrings[(string)lblNumSpellsThatCountAsLand.Tag];

			// Deck Box
			gbDeckBox.Text = Settings.UIStrings[(string)gbDeckBox.Tag];
			rbUseDeckImage.Text = Settings.UIStrings[(string)rbUseDeckImage.Tag];
			rbLoadImage.Text = Settings.UIStrings[(string)rbLoadImage.Tag];
			rbBuildImage.Text = Settings.UIStrings[(string)rbBuildImage.Tag];
			gbDeckLocation.Text = Settings.UIStrings[(string)gbDeckLocation.Tag];
			gbDeckSize.Text = Settings.UIStrings[(string)gbDeckSize.Tag];

			// AI Personality Box
			gbPersonality.Text = Settings.UIStrings[(string)gbPersonality.Tag];
			cboPersonality.Refresh();
			cmdPersonalityEditName.Text = Settings.UIStrings[(string)cmdPersonalityEditName.Tag];
			lblMusic.Text = Settings.UIStrings[(string)lblMusic.Tag];
			rbUseThisImageSmall.Text = Settings.UIStrings[(string)rbUseThisImageSmall.Tag];
			rbLoadImageSmall.Text = Settings.UIStrings[(string)rbLoadImageSmall.Tag];
			rbBuildImageSmall.Text = Settings.UIStrings[(string)rbBuildImageSmall.Tag];
			gbSmallLocation.Text = Settings.UIStrings[(string)gbSmallLocation.Tag];
			gbSmallSize.Text = Settings.UIStrings[(string)gbSmallSize.Tag];
			rbUseThisImageLarge.Text = Settings.UIStrings[(string)rbUseThisImageLarge.Tag];
			rbLoadImageLarge.Text = Settings.UIStrings[(string)rbLoadImageLarge.Tag];
			rbBuildImageLarge.Text = Settings.UIStrings[(string)rbBuildImageLarge.Tag];
			gbLargeLocation.Text = Settings.UIStrings[(string)gbLargeLocation.Tag];
			gbLargeSize.Text = Settings.UIStrings[(string)gbLargeSize.Tag];

			// Apply & Cancel
			cmdApply.Text = Settings.UIStrings[(string)cmdApply.Tag];
			cmdCancel.Text = Settings.UIStrings[(string)cmdCancel.Tag];
		}

		private void cmdCancel_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void cmdApply_Click(object sender, EventArgs e)
		{
			// Save off our values.
			m_dkInfo.Availability = ((KeyValuePair<DeckAvailability, string>)cboAvailability.SelectedItem).Key;
			// If they apply before they have added any cards to the deck then it will error out trying to get the card filename so check for this case.
			if (cboPreviewCard.SelectedItem != null)
				m_dkInfo.PreviewCard = ((DeckCard)cboPreviewCard.SelectedItem).Card.Filename;
			m_dkInfo.Uid = (int)numUid.Value;
			m_dkInfo.CreatureSize = ValueFromRadioButtonPanel(pnlSize);
			m_dkInfo.DeckSpeed = ValueFromRadioButtonPanel(pnlSpeed);
			m_dkInfo.Flexibility = ValueFromRadioButtonPanel(pnlFlexibility);
			m_dkInfo.Synergy = ValueFromRadioButtonPanel(pnlSynergy);
			m_dkInfo.IgnoreCmcOver = (int)numIgnoreCmcOver.Value;
			m_dkInfo.MinForests = (int)numMinForests.Value;
			m_dkInfo.MinIslands = (int)numMinIslands.Value;
			m_dkInfo.MinMountains = (int)numMinMountains.Value;
			m_dkInfo.MinPlains = (int)numMinPlains.Value;
			m_dkInfo.MinSwamps = (int)numMinSwamps.Value;
			m_dkInfo.NumberOfSpellsThatCountAsLands = (int)numSpellsThatCountAsLand.Value;
			// If they apply with a null image then trying to make a new bitmap from it will error out so check for that.
			if (m_dkInfo.DeckBoxImage != null)
			{
				m_dkInfo.DeckBoxImage.Dispose();
				m_dkInfo.DeckBoxImage = null;
			}
			if (picDeckBox.Image != null)
				m_dkInfo.DeckBoxImage = new Bitmap(picDeckBox.Image);
			// Set our personality.
			m_dkInfo.Personality = m_apPersonality;
			m_dkInfo.Edited = true;

			this.Close();
		}

		private string ValueFromRadioButtonPanel(Panel pnlButtons)
		{
			string strReturn = "?";	// In case we can't figure it out which shouldn't be the case.

			foreach (Control ctrl in pnlButtons.Controls)
			{
				if (ctrl is RadioButton)
				{
					RadioButton rbButton = ctrl as RadioButton;
					if (rbButton.Checked)
					{
						strReturn = (string)rbButton.Tag;
					}
				}
			}

			return strReturn;
		}

		private void cmdGetFirstAvailableUid_Click(object sender, EventArgs e)
		{
			try
			{
				int nId = m_isScheme.GetNextAvailableId(m_dicUsedIds);
				if (nId > -1)
					numUid.Value = nId;
				else
				{
					// No available ids given the current scheme settings.
					MessageBox.Show(Settings.UIStrings["NO_AVAILABLE_IDS_MESSAGE"], Settings.UIStrings["NO_AVAILABLE_IDS_CAPTION"], MessageBoxButtons.OK, MessageBoxIcon.Warning);
				}
			}
			catch (Exception)
			{
				// If we catch an exception then there is probably something wrong with the scheme settings.
				MessageBox.Show(Settings.UIStrings["PROBLEM_WITH_SCHEME_SETTINGS_MESSAGE"], Settings.UIStrings["PROBLEM_WITH_SCHEME_SETTINGS_CAPTION"], MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void numUid_ValueChanged(object sender, EventArgs e)
		{
			try
			{
				lblFinalUid.Text = m_isScheme.GetDeckId((int)numUid.Value).ToString();
			}
			catch (Exception)
			{
				lblFinalUid.Text = Settings.UIStrings["INVALID"];
			}
		}

		private void rbUseDeckImage_CheckedChanged(object sender, EventArgs e)
		{
			if ((rbUseDeckImage.Checked) && (!m_bChangingToLast))
			{
				if (picDeckBox.Image != null)
					picDeckBox.Image.Dispose();
				if (m_dkInfo.DeckBoxImage != null)
					picDeckBox.Image = new Bitmap(m_dkInfo.DeckBoxImage);
				else
					picDeckBox.Image = null;
				m_bBuildDeckImage = false;
				m_rbDeckLastChecked = rbUseDeckImage;
			}
		}

		private void rbLoadImage_CheckedChanged(object sender, EventArgs e)
		{
			if ((rbLoadImage.Checked) && (!m_bChangingToLast))
			{
				Bitmap bmpChosen = ChooseImage();
				if (bmpChosen != null)
				{
					if (picDeckBox.Image != null)
						picDeckBox.Image.Dispose();
					picDeckBox.Image = bmpChosen;
					m_rbDeckLastChecked = rbLoadImage;
				}
				else
				{
					// They either canceled or picked a bad file, either way we go back to where we were last.
					m_bChangingToLast = true;
					m_rbDeckLastChecked.Checked = true;
					m_bChangingToLast = false;
				}
			}
		}

		private void rbBuildImage_CheckedChanged(object sender, EventArgs e)
		{
			if ((rbBuildImage.Checked) && (!m_bChangingToLast))
			{
				Bitmap bmpChosen = ChooseImage();
				if (bmpChosen != null)
				{
					m_ibDeckBox.LoadedImage = bmpChosen;
					InitialDeckImageAdjust();
					if (picDeckBox.Image != null)
						picDeckBox.Image.Dispose();
					picDeckBox.Image = m_ibDeckBox.BuildImage();

					// We've successfully built an initial image so go ahead and enable our controls.
					m_rbDeckLastChecked = rbBuildImage;
					m_bBuildDeckImage = true;
					ChangeControlEnabled(scDeckLocationSize, true);
				}
				else
				{
					// We've successfully built an initial image so go ahead and enable our controls.
					m_rbDeckLastChecked = rbBuildImage;
					m_bBuildDeckImage = true;
					ChangeControlEnabled(scDeckLocationSize, true);
				}
			}
			else if ((m_bChangingToLast) && (m_rbDeckLastChecked == rbBuildImage))
			{
				// We were the last checked so we go back to here and re-enable build mode.
				m_bBuildDeckImage = true;
				ChangeControlEnabled(scDeckLocationSize, true);
			}
			else
			{
				// Disable our controls.
				m_bBuildDeckImage = false;
				ChangeControlEnabled(scDeckLocationSize, false);
			}
		}

		private void ChangeControlEnabled(Control ctrlControl, bool bEnabled)
		{
			ctrlControl.Enabled = bEnabled;
			if ((ctrlControl.Controls != null) && (ctrlControl.Controls.Count > 0))
			{
				foreach (Control ctrl in ctrlControl.Controls)
				{
					ChangeControlEnabled(ctrl, bEnabled);
				}
			}
		}

		private void InitialDeckImageAdjust()
		{
			m_ibDeckBox.InitialImageAdjust();

			// Update our controls (without screwing this up)
			m_bProgramaticAdjust = true;
			numDeckX.Value = m_ibDeckBox.AdjustedRect.X;
			numDeckY.Value = m_ibDeckBox.AdjustedRect.Y;
			numDeckWidth.Value = m_ibDeckBox.AdjustedRect.Width;
			numDeckHeight.Value = m_ibDeckBox.AdjustedRect.Height;
			m_bProgramaticAdjust = false;
		}

		private Bitmap ChooseImage()
		{
			Bitmap bmpReturn = null;

			OpenFileDialog ofdLoad = new OpenFileDialog();
			ofdLoad.AddExtension = false;
			ofdLoad.AutoUpgradeEnabled = true;
			ofdLoad.CheckFileExists = true;
			ofdLoad.CheckPathExists = true;
			ofdLoad.DereferenceLinks = true;
			ofdLoad.Filter = Settings.UIStrings["LOAD_IMAGE_FILTER"];
			ofdLoad.FilterIndex = 1;
			ofdLoad.Multiselect = false;
			ofdLoad.Title = Settings.UIStrings["LOAD_IMAGE_CAPTION"];
			DialogResult drResult = ofdLoad.ShowDialog();
			if (drResult == DialogResult.OK)
			{
				// We should have a valid filename and extension so time to load the image.
				try
				{
					if (Path.GetExtension(ofdLoad.FileName).Equals(".tdx", StringComparison.OrdinalIgnoreCase))
					{
						TdxWrapper twLoaded = new TdxWrapper();
						twLoaded.LoadTdx(ofdLoad.FileName);
						bmpReturn = new Bitmap(twLoaded.Image);
						twLoaded.Dispose();
					}
					else
					{
						Bitmap bmpLoading = new Bitmap(ofdLoad.FileName);
						bmpReturn = bmpLoading;
					}
				}
				catch (Exception e)
				{
					// We don't really care what happens here, we'll just treat it as though the user canceled.
					//	But we will report the error anyway.
					Settings.ReportError(e, ErrorPriority.Low, "Failed to load image: " + ofdLoad.FileName);
				}
			}

			return bmpReturn;
		}

		private void numDeckX_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildDeckImage) && (!m_bProgramaticAdjust))
			{
				m_ibDeckBox.AdjustedRect.X = (int)numDeckX.Value;
				picDeckBox.Image = m_ibDeckBox.BuildImage();
			}
		}

		private void numDeckY_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildDeckImage) && (!m_bProgramaticAdjust))
			{
				m_ibDeckBox.AdjustedRect.Y = (int)numDeckY.Value;
				picDeckBox.Image = m_ibDeckBox.BuildImage();
			}
		}

		private void numDeckWidth_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildDeckImage) && (!m_bProgramaticAdjust))
			{
				m_ibDeckBox.AdjustedRect.Width = (int)numDeckWidth.Value;
				picDeckBox.Image = m_ibDeckBox.BuildImage();
			}
		}

		private void numDeckHeight_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildDeckImage) && (!m_bProgramaticAdjust))
			{
				m_ibDeckBox.AdjustedRect.Height = (int)numDeckHeight.Value;
				picDeckBox.Image = m_ibDeckBox.BuildImage();
			}
		}

		private void picDeckBox_MouseDown(object sender, MouseEventArgs e)
		{
			if (m_bBuildDeckImage)
				m_ibDeckBox.MouseDownHandler(e);
		}

		private void picDeckBox_MouseMove(object sender, MouseEventArgs e)
		{
			if (m_bBuildDeckImage)
			{
				picDeckBox.Image = m_ibDeckBox.MouseMoveHandler(e);
				// And update the location numeric up downs.
				lock (numDeckX)
				{
					m_bProgramaticAdjust = true;
					numDeckX.Value = m_ibDeckBox.AdjustedRect.X;
					numDeckY.Value = m_ibDeckBox.AdjustedRect.Y;
					m_bProgramaticAdjust = false;
				}
			}
		}

		private void picDeckBox_MouseUp(object sender, MouseEventArgs e)
		{
			if (m_bBuildDeckImage)
				m_ibDeckBox.MouseUpHandler(e);
		}

		private void picDeckBox_MouseWheel(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if (m_bBuildDeckImage)
			{
				picDeckBox.Image = m_ibDeckBox.MouseWheelHandler(e);
				lock (numDeckX)
				{
					m_bProgramaticAdjust = true;
					numDeckX.Value = m_ibDeckBox.AdjustedRect.X;
					numDeckY.Value = m_ibDeckBox.AdjustedRect.Y;
					numDeckWidth.Value = m_ibDeckBox.AdjustedRect.Width;
					numDeckHeight.Value = m_ibDeckBox.AdjustedRect.Height;
					m_bProgramaticAdjust = false;
				}
			}
		}

		private Control GetFocusedControl(Control ctrl)
		{
			Control ctrlReturn = null;

			if (ctrl.Focused)
				ctrlReturn = ctrl;
			else if ((ctrl.Controls != null) && (ctrl.Controls.Count > 0))
			{
				foreach (Control ctrlSub in ctrl.Controls)
				{
					ctrlReturn = GetFocusedControl(ctrlSub);
					if (ctrlReturn != null)
						break;
				}
			}

			return ctrlReturn;
		}

		private void picDeckBox_MouseEnter(object sender, EventArgs e)
		{
			if (m_bBuildDeckImage)
			{
				// Take focus so that we can get the mouse wheel events.
				picDeckBox.Focus();
			}
		}

		private void DeckInformation_FormClosing(object sender, FormClosingEventArgs e)
		{
			Settings.SaveSetting("DeckInfoPosition", new Rectangle(this.Location, this.Size));
		}

		private void cboPersonality_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (!m_bLoadingPersonality)
			{
				if (cboPersonality.SelectedItem != null)
				{
					if (cboPersonality.SelectedItem != m_apPersonality)
					{
						m_apPersonality = cboPersonality.SelectedItem as AiPersonality;
						RefreshPersonality();
					}
				}
			}
		}

		private void cmdPersonalityEditName_Click(object sender, EventArgs e)
		{
			Dictionary<string, string> dicTemp = new Dictionary<string, string>(m_apPersonality.Name);
			EditString frmEdit = new EditString(dicTemp, cmdPersonalityEditName.Text);
			if (frmEdit.ShowDialog() == DialogResult.OK)
			{
				PrepPersonalityForEditing();
				m_apPersonality.Name = dicTemp;
				lblPersonalityName.Text = m_apPersonality.LocalizedName;
				cboPersonality.Refresh();
			}
			frmEdit.Dispose();
		}

		private void cboMusic_SelectedIndexChanged(object sender, EventArgs e)
		{
			if ((!m_bLoadingPersonality) && (m_apPersonality != null) && (cboMusic.SelectedIndex > -1))
			{
				PrepPersonalityForEditing();
				m_apPersonality.MusicMix = (string)cboMusic.SelectedItem;
			}
		}

		private void rbUseThisImageSmall_CheckedChanged(object sender, EventArgs e)
		{
			if ((rbUseThisImageSmall.Checked) && (!m_bChangingToLast))
			{
				if (picPersonalitySmall.Image != null)
					picPersonalitySmall.Image.Dispose();
				if (m_apPersonality.SmallAvatarImage != null)
					picPersonalitySmall.Image = new Bitmap(m_apPersonality.SmallAvatarImage);
				else
					picPersonalitySmall.Image = null;
				m_bBuildSmallAvatar = false;
				m_rbSmallAvatarLastChecked = rbUseThisImageSmall;
			}
		}

		private void rbLoadImageSmall_CheckedChanged(object sender, EventArgs e)
		{
			if ((rbLoadImageSmall.Checked) && (!m_bChangingToLast))
			{
				Bitmap bmpChosen = ChooseImage();
				if (bmpChosen != null)
				{
					PrepPersonalityForEditing();
					m_apPersonality.SmallAvatarImage = bmpChosen;
					if (picPersonalitySmall.Image != null)
						picPersonalitySmall.Image.Dispose();
					picPersonalitySmall.Image = m_apPersonality.SmallAvatarImage;
					m_rbSmallAvatarLastChecked = rbLoadImageSmall;
				}
				else
				{
					// They either canceled or picked a bad file, either way we go back to where we were last.
					m_bChangingToLast = true;
					m_rbSmallAvatarLastChecked.Checked = true;
					m_bChangingToLast = false;
				}
			}
		}

		private void rbBuildImageSmall_CheckedChanged(object sender, EventArgs e)
		{
			if ((rbBuildImageSmall.Checked) && (!m_bChangingToLast))
			{
				Bitmap bmpChosen = ChooseImage();
				if (bmpChosen != null)
				{
					PrepPersonalityForEditing();
					m_ibSmallAvatar.LoadedImage = bmpChosen;
					InitialSmallAvatarAdjust();
					m_apPersonality.SmallAvatarImage = m_ibSmallAvatar.BuildImage();
					if (picPersonalitySmall.Image != null)
						picPersonalitySmall.Image.Dispose();
					picPersonalitySmall.Image = m_apPersonality.SmallAvatarImage;

					// We've successfully built an initial image so go ahead and enable our controls.
					m_rbSmallAvatarLastChecked = rbBuildImageSmall;
					m_bBuildSmallAvatar = true;
					ChangeControlEnabled(scSmallLocationSize, true);
				}
				else
				{
					// They either canceled or picked a bad file, either way we go back to where we were last.
					m_bChangingToLast = true;
					m_rbSmallAvatarLastChecked.Checked = true;
					m_bChangingToLast = false;
				}
			}
			else if ((m_bChangingToLast) && (m_rbSmallAvatarLastChecked == rbBuildImageSmall))
			{
				// We were the last checked so we go back to here and re-enable build mode.
				m_bBuildSmallAvatar = true;
				ChangeControlEnabled(scSmallLocationSize, true);
			}
			else
			{
				// Disable our controls.
				m_bBuildSmallAvatar = false;
				ChangeControlEnabled(scSmallLocationSize, false);
			}
		}

		private void InitialSmallAvatarAdjust()
		{
			m_ibSmallAvatar.InitialImageAdjust();

			// Update our controls (without screwing this up)
			m_bProgramaticAdjust = true;
			numSmallX.Value = m_ibSmallAvatar.AdjustedRect.X;
			numSmallY.Value = m_ibSmallAvatar.AdjustedRect.Y;
			numSmallWidth.Value = m_ibSmallAvatar.AdjustedRect.Width;
			numSmallHeight.Value = m_ibSmallAvatar.AdjustedRect.Height;
			m_bProgramaticAdjust = false;
		}

		private void numSmallX_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildSmallAvatar) && (!m_bProgramaticAdjust))
			{
				m_ibSmallAvatar.AdjustedRect.X = (int)numSmallX.Value;
				m_apPersonality.SmallAvatarImage = m_ibSmallAvatar.BuildImage();
				picPersonalitySmall.Image = m_apPersonality.SmallAvatarImage;
			}
		}

		private void numSmallY_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildSmallAvatar) && (!m_bProgramaticAdjust))
			{
				m_ibSmallAvatar.AdjustedRect.Y = (int)numSmallY.Value;
				m_apPersonality.SmallAvatarImage = m_ibSmallAvatar.BuildImage();
				picPersonalitySmall.Image = m_apPersonality.SmallAvatarImage;
			}
		}

		private void numSmallWidth_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildSmallAvatar) && (!m_bProgramaticAdjust))
			{
				m_ibSmallAvatar.AdjustedRect.Width = (int)numSmallWidth.Value;
				m_apPersonality.SmallAvatarImage = m_ibSmallAvatar.BuildImage();
				picPersonalitySmall.Image = m_apPersonality.SmallAvatarImage;
			}
		}

		private void numSmallHeight_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildSmallAvatar) && (!m_bProgramaticAdjust))
			{
				m_ibSmallAvatar.AdjustedRect.Height = (int)numSmallHeight.Value;
				m_apPersonality.SmallAvatarImage = m_ibSmallAvatar.BuildImage();
				picPersonalitySmall.Image = m_apPersonality.SmallAvatarImage;
			}
		}

		private void picPersonalitySmall_MouseDown(object sender, MouseEventArgs e)
		{
			if (m_bBuildSmallAvatar)
				m_ibSmallAvatar.MouseDownHandler(e);
		}

		private void picPersonalitySmall_MouseMove(object sender, MouseEventArgs e)
		{
			if (m_bBuildSmallAvatar)
			{
				m_apPersonality.SmallAvatarImage = m_ibSmallAvatar.MouseMoveHandler(e);
				picPersonalitySmall.Image = m_apPersonality.SmallAvatarImage;
				// And update the location numeric up downs.
				lock (numSmallX)
				{
					m_bProgramaticAdjust = true;
					numSmallX.Value = m_ibSmallAvatar.AdjustedRect.X;
					numSmallY.Value = m_ibSmallAvatar.AdjustedRect.Y;
					m_bProgramaticAdjust = false;
				}
			}
		}

		private void picPersonalitySmall_MouseUp(object sender, MouseEventArgs e)
		{
			if (m_bBuildSmallAvatar)
				m_ibSmallAvatar.MouseUpHandler(e);
		}

		private void picPersonalitySmall_MouseWheel(object sender, MouseEventArgs e)
		{
			if (m_bBuildSmallAvatar)
			{
				m_apPersonality.SmallAvatarImage = m_ibSmallAvatar.MouseWheelHandler(e);
				picPersonalitySmall.Image = m_apPersonality.SmallAvatarImage;
				lock (numSmallX)
				{
					m_bProgramaticAdjust = true;
					numSmallX.Value = m_ibSmallAvatar.AdjustedRect.X;
					numSmallY.Value = m_ibSmallAvatar.AdjustedRect.Y;
					numSmallWidth.Value = m_ibSmallAvatar.AdjustedRect.Width;
					numSmallHeight.Value = m_ibSmallAvatar.AdjustedRect.Height;
					m_bProgramaticAdjust = false;
				}
			}
		}

		private void picPersonalitySmall_MouseEnter(object sender, EventArgs e)
		{
			if (m_bBuildSmallAvatar)
			{
				// Take focus so that we can get the mouse wheel events.
				picPersonalitySmall.Focus();
			}
		}

		private void rbUseThisImageLarge_CheckedChanged(object sender, EventArgs e)
		{
			if ((rbUseThisImageLarge.Checked) && (!m_bChangingToLast))
			{
				if (picPersonalityLarge.Image != null)
					picPersonalityLarge.Image.Dispose();
				if (m_apPersonality.LargeAvatarImage != null)
					picPersonalityLarge.Image = new Bitmap(m_apPersonality.LargeAvatarImage);
				else
					picPersonalityLarge.Image = null;
				m_bBuildLargeAvatar = false;
				m_rbLargeAvatarLastChecked = rbUseThisImageLarge;
			}
		}

		private void rbLoadImageLarge_CheckedChanged(object sender, EventArgs e)
		{
			if ((rbLoadImageLarge.Checked) && (!m_bChangingToLast))
			{
				Bitmap bmpChosen = ChooseImage();
				if (bmpChosen != null)
				{
					PrepPersonalityForEditing();
					if (picPersonalityLarge.Image != null)
						picPersonalityLarge.Image.Dispose();
					m_apPersonality.LargeAvatarImage = bmpChosen;
					picPersonalityLarge.Image = m_apPersonality.LargeAvatarImage;
					m_rbLargeAvatarLastChecked = rbLoadImageLarge;
				}
				else
				{
					// They either canceled or picked a bad file, either way we go back to where we were last.
					m_bChangingToLast = true;
					m_rbLargeAvatarLastChecked.Checked = true;
					m_bChangingToLast = false;
				}
			}
		}

		private void rbBuildImageLarge_CheckedChanged(object sender, EventArgs e)
		{
			if ((rbBuildImageLarge.Checked) && (!m_bChangingToLast))
			{
				Bitmap bmpChosen = ChooseImage();
				if (bmpChosen != null)
				{
					PrepPersonalityForEditing();
					m_ibLargeAvatar.LoadedImage = bmpChosen;
					InitialLargeAvatarAdjust();
					m_apPersonality.LargeAvatarImage = m_ibLargeAvatar.BuildImage();
					if (picPersonalityLarge.Image != null)
						picPersonalityLarge.Image.Dispose();
					picPersonalityLarge.Image = m_apPersonality.LargeAvatarImage;

					// We've successfully built an initial image so go ahead and enable our controls.
					m_rbLargeAvatarLastChecked = rbBuildImageLarge;
					m_bBuildLargeAvatar = true;
					ChangeControlEnabled(scLargeLocationSize, true);
				}
				else
				{
					// They either canceled or picked a bad file, either way we go back to where we were last.
					m_bChangingToLast = true;
					m_rbLargeAvatarLastChecked.Checked = true;
					m_bChangingToLast = false;
				}
			}
			else if ((m_bChangingToLast) && (m_rbLargeAvatarLastChecked == rbBuildImageLarge))
			{
				// We were the last checked so we go back to here and re-enable build mode.
				m_bBuildLargeAvatar = true;
				ChangeControlEnabled(scLargeLocationSize, true);
			}
			else
			{
				// Disable our controls.
				m_bBuildLargeAvatar = false;
				ChangeControlEnabled(scLargeLocationSize, false);
			}
		}

		private void InitialLargeAvatarAdjust()
		{
			m_ibLargeAvatar.InitialImageAdjust();

			// Update our controls (without screwing this up)
			m_bProgramaticAdjust = true;
			numLargeX.Value = m_ibLargeAvatar.AdjustedRect.X;
			numLargeY.Value = m_ibLargeAvatar.AdjustedRect.Y;
			numLargeWidth.Value = m_ibLargeAvatar.AdjustedRect.Width;
			numLargeHeight.Value = m_ibLargeAvatar.AdjustedRect.Height;
			m_bProgramaticAdjust = false;
		}

		private void numLargeX_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildLargeAvatar) && (!m_bProgramaticAdjust))
			{
				m_ibLargeAvatar.AdjustedRect.X = (int)numLargeX.Value;
				m_apPersonality.LargeAvatarImage = m_ibLargeAvatar.BuildImage();
				picPersonalityLarge.Image = m_apPersonality.LargeAvatarImage;
			}
		}

		private void numLargeY_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildLargeAvatar) && (!m_bProgramaticAdjust))
			{
				m_ibLargeAvatar.AdjustedRect.Y = (int)numLargeY.Value;
				m_apPersonality.LargeAvatarImage = m_ibLargeAvatar.BuildImage();
				picPersonalityLarge.Image = m_apPersonality.LargeAvatarImage;
			}
		}

		private void numLargeWidth_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildLargeAvatar) && (!m_bProgramaticAdjust))
			{
				m_ibLargeAvatar.AdjustedRect.Width = (int)numLargeWidth.Value;
				m_apPersonality.LargeAvatarImage = m_ibLargeAvatar.BuildImage();
				picPersonalityLarge.Image = m_apPersonality.LargeAvatarImage;
			}
		}

		private void numLargeHeight_ValueChanged(object sender, EventArgs e)
		{
			// We're not going to bother if we aren't building an image or we got here through programatic adjustment.
			if ((m_bBuildLargeAvatar) && (!m_bProgramaticAdjust))
			{
				m_ibLargeAvatar.AdjustedRect.Height = (int)numLargeHeight.Value;
				m_apPersonality.LargeAvatarImage = m_ibLargeAvatar.BuildImage();
				picPersonalityLarge.Image = m_apPersonality.LargeAvatarImage;
			}
		}

		private void picPersonalityLarge_MouseDown(object sender, MouseEventArgs e)
		{
			if (m_bBuildLargeAvatar)
				m_ibLargeAvatar.MouseDownHandler(e);
		}

		private void picPersonalityLarge_MouseMove(object sender, MouseEventArgs e)
		{
			if (m_bBuildLargeAvatar)
			{
				m_apPersonality.LargeAvatarImage = m_ibLargeAvatar.MouseMoveHandler(e);
				picPersonalityLarge.Image = m_apPersonality.LargeAvatarImage;
				// And update the location numeric up downs.
				lock (numLargeX)
				{
					m_bProgramaticAdjust = true;
					numLargeX.Value = m_ibLargeAvatar.AdjustedRect.X;
					numLargeY.Value = m_ibLargeAvatar.AdjustedRect.Y;
					m_bProgramaticAdjust = false;
				}
			}
		}

		private void picPersonalityLarge_MouseUp(object sender, MouseEventArgs e)
		{
			if (m_bBuildLargeAvatar)
				m_ibLargeAvatar.MouseUpHandler(e);
		}

		private void picPersonalityLarge_MouseEnter(object sender, EventArgs e)
		{
			if (m_bBuildLargeAvatar)
			{
				// Take focus so that we can get the mouse wheel events.
				picPersonalityLarge.Focus();
			}
		}

		private void picPersonalityLarge_MouseWheel(object sender, MouseEventArgs e)
		{
			if (m_bBuildLargeAvatar)
			{
				m_apPersonality.LargeAvatarImage = m_ibLargeAvatar.MouseWheelHandler(e);
				picPersonalityLarge.Image = m_apPersonality.LargeAvatarImage;
				lock (numLargeX)
				{
					m_bProgramaticAdjust = true;
					numLargeX.Value = m_ibLargeAvatar.AdjustedRect.X;
					numLargeY.Value = m_ibLargeAvatar.AdjustedRect.Y;
					numLargeWidth.Value = m_ibLargeAvatar.AdjustedRect.Width;
					numLargeHeight.Value = m_ibLargeAvatar.AdjustedRect.Height;
					m_bProgramaticAdjust = false;
				}
			}
		}
	}
}
